# -*- coding: utf8 -*-

import json


def dummy_encoder(obj):
    return obj


class DummyDecoder(json.JSONDecoder):

    def __init__(self):
        json.JSONDecoder.__init__(self, object_hook=self.dict_to_object)

    def dict_to_object(self, d):
        for k, v in d.items():
            d[k] = 'DUMMY'
        return d


class TestRegistry(object):

    def __init__(self, settings=None):

        if settings is None:
            self.settings = {}
        else:
            self.settings = settings

        self.rawes = None

    def queryUtility(self, iface, name):
        return self.rawes

    def registerUtility(self, rawes, iface, name):
        self.rawes = rawes



# -*- coding: utf8 -*-

import rawes

from pyramid.settings import aslist

from zope.interface import Interface

from rawes.encoders import encode_date_optional_time

from pyramid.path import (
    DottedNameResolver
)

import warnings


class IRawes(Interface):
    pass


def _parse_settings(settings):

    rawes_args = {}
    defaults = {
        'url': 'http://localhost:9200',
        'timeout': 30,
        'path': '',
        'json_encoder': encode_date_optional_time,
    }

    rawes_args = defaults.copy()

    # set string settings
    for short_key_name in ('path',):
        key_name = 'rawes.%s' % (short_key_name,)
        if key_name in settings:
            rawes_args[short_key_name] = \
                settings.get(key_name, defaults.get(short_key_name))

    # set list settings
    for short_key_name in ('url',):
        key_name = 'rawes.%s' % (short_key_name,)
        if key_name in settings:
            rawes_args[short_key_name] = \
                (aslist(settings.get(key_name, defaults.get(short_key_name)))
		        if len(aslist(settings.get(key_name, defaults.get(short_key_name)))) > 1
		        else settings.get(key_name, defaults.get(short_key_name)).strip())

    # integer settings
    for short_key_name in ('timeout',):
        key_name = 'rawes.%s' % (short_key_name,)
        if key_name in settings:
            rawes_args[short_key_name] = \
                int(settings.get(key_name, defaults.get(short_key_name)))

    # function settings
    for short_key_name in ('json_encoder',):
        key_name = 'rawes.%s' % (short_key_name,)
        r = DottedNameResolver()
        if key_name in settings:
            rawes_args[short_key_name] = \
                r.resolve(settings.get(key_name))
    for short_key_name in ('json_decoder',):
        key_name = 'rawes.%s' % (short_key_name,)
        r = DottedNameResolver()
        if key_name in settings:
            rawes_args[short_key_name] = \
                r.resolve(settings.get(key_name))().decode

    # removed settings
    for short_key_name in ('connection_type', 'except_on_error'):
        key_name = 'rawes.%s' % (short_key_name,)
        if key_name in settings:
            warnings.warn(
                '%s is no longer supported, please remove from your settings.',
                UserWarning
            )

    return rawes_args


def _build_rawes(registry):
    """
    Build a RawES connection to Elastic Search and add it to the registry.
    """
    ES = registry.queryUtility(IRawes, 'rawes')
    if ES is not None:
        return ES

    settings = registry.settings
    rawes_args = _parse_settings(settings)

    ES = rawes.Elastic(**rawes_args)

    registry.registerUtility(ES, IRawes, 'rawes')
    return registry.queryUtility(IRawes, 'rawes')


def get_rawes(registry):
    """
    Get the RawES connection.
    """
    #Argument might be a config or request
    regis = getattr(registry, 'registry', None)
    if regis is None:
        regis = registry
    return regis.queryUtility(IRawes, 'rawes')


def includeme(config):
    _build_rawes(config.registry)
    config.add_directive('get_rawes', get_rawes)



import os

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()
CHANGES = open(os.path.join(here, 'HISTORY.rst')).read()

requires = [
    'pyramid',
    'rawes>=0.5.4'
    ]

setup(
    name='pyramid_rawes',
    version='0.5.1',
    license='MIT',
    description='Rawes bindings for pyramid',
    long_description=README + '\n\n' +  CHANGES,
    classifiers=[
        'Intended Audience :: Developers',
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Framework :: Pyramid',
        'Topic :: Internet :: WWW/HTTP :: Indexing/Search'
    ],
    author='Koen Van Daele',
    author_email='koen_van_daele@telenet.be',
    url='https://github.com/koenedaele/pyramid_rawes',
    keywords='pyramid elastic search rawes',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires = requires,
    test_suite='pyramid_rawes',
)



# -*- coding: utf8 -*-

from pyramid import testing

from pyramid_rawes import (
    IRawes,
    get_rawes,
    _build_rawes,
    includeme,
    _parse_settings
)

import rawes
import warnings
import unittest

from .fixtures import (
    dummy_encoder,
    TestRegistry
)


class TestGetAndBuild(unittest.TestCase):

    def test_get_rawes(self):
        r = TestRegistry()
        ES = rawes.Elastic(url='http://localhost:9200')
        r.registerUtility(ES, IRawes, 'rawes')
        ES2 = get_rawes(r)
        self.assertEqual(ES, ES2)

    def test_build_rawes_already_exists(self):
        r = TestRegistry()
        ES = rawes.Elastic('http://localhost:9200')
        r.registerUtility(ES, IRawes, 'rawes')
        ES2 = _build_rawes(r)
        self.assertEqual(ES, ES2)

    def test_build_rawes_default_settings(self):
        r = TestRegistry()
        ES = _build_rawes(r)
        self.assertIsInstance(ES, rawes.Elastic)
        self.assertEqual(1, len(ES.connection_pool.connections))

    def test_build_rawes_custom_settings(self):
        settings = {
            'rawes.url': 'http://elastic.search.org:9200',
            'rawes.path': '/search',
            'rawes.timeout': 123
        }
        r = TestRegistry(settings)
        ES = _build_rawes(r)
        self.assertIsInstance(ES, rawes.Elastic)
        self.assertEqual(1, len(ES.connection_pool.connections))

    def test_build_rawes_url_list_settings(self):
        settings = {
            'rawes.url': 'http://el1.search.org:9200\nhttp://el2.search.org:9200',
            'rawes.path': '/search',
            'rawes.timeout': 123
        }
        r = TestRegistry(settings)
        ES = _build_rawes(r)
        self.assertIsInstance(ES, rawes.Elastic)
        self.assertEqual(2, len(ES.connection_pool.connections))


class TestSettings(unittest.TestCase):

    def _assert_contains_all_keys(self, args):
        self.assertIn('url', args)
        self.assertIn('path', args)
        self.assertIn('timeout', args)

    def test_get_default_settings(self):
        settings = {}
        args = _parse_settings(settings)
        self._assert_contains_all_keys(args)

    def test_get_some_settings(self):
        settings = {
            'rawes.url': 'http://elastic.search.org:9200',
            'rawes.timeout': 123,
        }
        args = _parse_settings(settings)
        self._assert_contains_all_keys(args)
        self.assertEqual('http://elastic.search.org:9200', args['url'])
        self.assertEqual(123, args['timeout'])
        settings = {
                'rawes.url': 'http://elastic.search.org:9200\nhttp://elastic.search.org:9300',
        }
        args = _parse_settings(settings)
        self._assert_contains_all_keys(args)
        self.assertEqual(['http://elastic.search.org:9200', 'http://elastic.search.org:9300'], args['url'])

    def test_get_all_settings(self):
        settings = {
            'rawes.url': 'http://elastic.search.org:9200',
            'rawes.path': '/search',
            'rawes.timeout': 123,
        }
        args = _parse_settings(settings)
        self._assert_contains_all_keys(args)
        self.assertEqual(123, args['timeout'])

    def test_get_dotted_function_settings(self):
        settings = {
            'rawes.json_encoder': 'tests.fixtures.dummy_encoder'
        }
        args = _parse_settings(settings)
        self.assertEqual(dummy_encoder, args['json_encoder'])

    def test_list_settings(self):
        settings = {
            'rawes.url': 'http://el1.search.org:9200\nhttp://el2.search.org:9200'
        }
        args = _parse_settings(settings)
        self.assertEqual(
            [
                'http://el1.search.org:9200',
                'http://el2.search.org:9200'
            ], 
            args['url']
        )

    def test_get_unsupported_settings(self):
        settings = {
            'rawes.except_on_error': False
        }
        with warnings.catch_warnings(record=True) as w:
            args = _parse_settings(settings)
            self.assertEqual(1, len(w))


class TestIncludeMe(unittest.TestCase):

    def setUp(self):
        self.config = testing.setUp()
        self.config.registry.settings['rawes.url'] = 'http://localhost:9300'
        self.config.registry.settings['rawes.json_encoder'] = 'tests.fixtures.dummy_encoder'
        self.config.registry.settings['rawes.json_decoder'] = 'tests.fixtures.DummyDecoder'

    def tearDown(self):
        del self.config

    def test_includeme(self):
        includeme(self.config)
        ES = self.config.registry.queryUtility(IRawes, 'rawes')
        self.assertIsInstance(ES, rawes.Elastic)
        self.assertEqual(1, len(ES.connection_pool.connections))
        self.assertEqual('test', ES.json_encoder('test'))
        self.assertEqual(dummy_encoder, ES.json_encoder)
        self.assertEqual(
            {'test': 'DUMMY'},
            ES.connection_pool.connections[0].kwargs['json_decoder']('{"test": 1}')
        )

    def test_directive_was_added(self):
        includeme(self.config)
        ES = self.config.get_rawes()
        self.assertIsInstance(ES, rawes.Elastic)
        self.assertEqual(1, len(ES.connection_pool.connections))
